@@ -1,8 +1,11 @@
 import os
+import warnings
+
 from collections import OrderedDict
 import numpy as np
 
 import astropy.units as u
+from astropy.utils.exceptions import AstropyWarning, AstropyUserWarning
 
 from . import utils
 
@@ -502,6 +505,11 @@ class Template():
                         
         elif arrays is not None:
             self.wave, self.flux = arrays[0]*1., arrays[1]*1.
+            if arrays[0].shape[0] != np.atleast_2d(arrays[1]).shape[1]:
+                raise ValueError("Array dimensions don't match: "+
+                                 f'arrays[0]: {arrays[0].shape}, '+
+                                 f'arrays[1]: {arrays[1].shape}, ')
+                
             if hasattr(self.flux, 'unit'):
                 self.flux_unit = self.flux.unit
                 
@@ -687,7 +695,7 @@ class Template():
         Parameters
         ----------
         velocity_smooth: float
-            Velocity smoothing factor, in km/s.
+            Velocity smoothing *sigma*, in km/s.
         
         in_place : bool
             Set internal ``flux`` array to the smoothed array.  If False, then
@@ -729,6 +737,141 @@ class Template():
                             redshifts=self.redshifts)
 
 
+    def to_observed_frame(self, z=0, scalar=1., extra_sigma=0, lsf_func=None, to_air=False, wavelengths=None, smoothspec_kwargs={'fftsmooth':False}, include_igm=True, clip_wavelengths=[4500,9400]):
+        """
+        Smooth and resample to observed-frame wavelengths, including an
+        optional Line Spread Function (LSF)
+                
+        Note that the smoothing is performed with 
+        `prospect.utils.smoothing.smoothspec <https://prospect.readthedocs.io/en/latest/api/utils_api.html>`_, 
+        which doesn't integrate precisely over "pixels" for spectral
+        resolutions that are similar to or less than the target smoothing
+        factor.
+        
+        Parameters
+        ----------
+        z : float
+            Target redshift.  Note that only the wavelength array is shifted
+            by ``(1+z)``.  The flux densities optionally include IGM
+            absorption (and dust from the ``redfunc`` attribute) but don't
+            include the ``fl_obs = fl_rest / (1+z)`` scaling.
+        
+        scalar : float, array
+            Scalar value or array with same dimensions as ``wave`` and 
+            ``flux`` attributes
+            
+        extra_sigma : float
+            Extra velocity dispersion (sigma, km/s) to add in quadrature with 
+            the MUSE LSF
+        
+        lsf_func : 'Bacon', function
+            Line Spread Function (LSF).  If ``'Bacon'``, then use the "UDF-10"
+            MUSE LSF from `Bacon et al. 2017 
+            <https://ui.adsabs.harvard.edu/abs/2017A%26A...608A...1B>`_ (Eq. 
+            8). Can also be a ``function`` that takes an argument of
+            wavelength in Angstroms and returns the LSF sigma, in Angstroms.
+            If neither of these, then only `extra_sigma` will be applied.
+            
+        to_air : bool
+            Apply vacuum-to-air conversion with `mpdaf.obj.vactoair <https://mpdaf.readthedocs.io/en/latest/api/mpdaf.obj.vactoair.html>`_
+        
+        wavelengths : array, None
+            Optional wavelength grid (observed frame) of the target output 
+            (e.g., MUSE) spectrum
+        
+        smoothspec_kwargs : dict
+            Extra keyword arguments to pass to the Prospector smoothing 
+            function `prospect.utils.smoothing.smoothspec <https://prospect.readthedocs.io/en/latest/api/utils_api.html>`_.  
+            When testing with very high resolution templates around a specific
+            wavelength, ``smoothspec_kwargs = {'fftsmooth':True}`` did not
+            always work as expected, so be careful with this option (which is
+            much faster).
+        
+        include_igm : bool
+            Include IGM absorption at indicated redshift
+        
+        clip_wavelengths : [float, float]
+            Trim the full observed-frame wavelength array before convolving.  
+            The defaults bracket the nominal MUSE range.
+            
+        Returns
+        -------
+        tobs : `~eazy.template.Template`
+            Smoothed and resampled `~eazy.template.Template` object
+            
+        """
+        from astropy.stats import gaussian_sigma_to_fwhm
+        from prospect.utils.smoothing import smoothspec
+        
+        wobs = self.wave*(1+z)
+        
+        if include_igm:
+            igmz = self.igm_absorption(z, pow=include_igm)
+        else:
+            igmz = 1.
+        
+        if to_air:
+            try:
+                from mpdaf.obj import vactoair
+                wobs = vactoair(wobs)
+            except ImportError:
+                msg = ("`to_air` requested but `from mpdaf.obj import " + 
+                       "vactoair` failed")
+                warnings.warn(msg, AstropyUserWarning)
+        
+        if clip_wavelengths is not None:
+            clip = wobs >= clip_wavelengths[0]
+            clip &= wobs <= clip_wavelengths[1]
+            
+            if clip.sum() == 0:
+                raise ValueError('No template wavelengths found in the '+
+                                 f'clipping range {clip_wavelengths} '+ 
+                                 'Angstroms')
+        else:
+            clip = wobs > 0
+            
+        if lsf_func in ['Bacon']:
+            # UDF-10 LSF from Bacon et al. 2017
+            bacon_lsf_fwhm = lambda w: 5.866e-8 * w**2 - 9.187e-4*w + 6.04
+            sig_ang = bacon_lsf_fwhm(wobs[clip]) / gaussian_sigma_to_fwhm
+            
+            lsf_sigma = sig_ang/wobs[clip]*3.e5
+            lsf_func_name = 'MUSE-LSF'
+            
+        elif hasattr(lsf_func, '__call__'):
+            lsf_sigma = lsf_func(wobs[clip])/wobs[clip]*3.e5
+            lsf_func_name = 'user'
+        
+        else:
+            lsf_sigma = 0.
+            lsf_func_name = None
+        
+        # Quadrature sum of LSF and extra velocities    
+        vel_sigma = np.sqrt(lsf_sigma**2 + extra_sigma**2)
+        
+        # In Angstroms
+        smooth_lambda = vel_sigma / 3.e5 * wobs[clip]
+        
+        # Do the smoothing
+        flux_smooth = smoothspec(wobs[clip], 
+                                 (self.flux_flam(z=z)*igmz*scalar)[clip], 
+                                 resolution=smooth_lambda, 
+                                 smoothtype='lsf', **smoothspec_kwargs)
+        
+        newname = self.name + f' z={z:.3f}' 
+        if lsf_func_name is not None:
+            newname += ' + ' + lsf_func_name
+        
+        if extra_sigma > 0:
+            newname += ' + {extra_sigma:.1f} km/s'
+            
+        tobs = Template(arrays=(wobs[clip], flux_smooth), 
+                        name=newname, resample_wave=wavelengths, 
+                        redshifts=[z])
+        
+        return tobs
+
+
     def resample(self, new_wave, z=0, in_place=True, return_array=False, interp_func=None):
         """
         Resample the template to a new wavelength grid
@@ -857,7 +1000,7 @@ class Template():
         return iz
 
 
-    def zscale(self, z, scalar=1, apply_igm=True):
+    def zscale(self, z, scalar=1, include_igm=True, **kwargs):
         """Redshift the template and multiply by a scalar.
 
         Parameters
@@ -867,18 +1010,25 @@ class Template():
 
         scalar : float or array
             Multiplicative factor.  Additional factor of 1./(1+z) is implicit.
-
+        
+        include_igm : bool
+            Include Inoue (2014) IGM absorption (also can be passed as 
+            ``apply_igm`` in ``kwargs``.)
+        
         Returns
         -------
         ztemp : `~eazy.templates.Template`
             Redshifted and scaled spectrum.
 
         """
-        if apply_igm:
-            igmz = self.igm_absorption(z, pow=apply_igm)
+        if 'apply_igm' in kwargs:
+            include_igm = kwargs['apply_igm']
+            
+        if include_igm:
+            igmz = self.igm_absorption(z, pow=include_igm)
         else:
             igmz = 1.
-
+        
         return Template(arrays=(self.wave*(1+z),
                                 self.flux_flam(z=z)*scalar/(1+z)*igmz), 
                         name=f'{self.name} z={z}')
@@ -956,8 +1106,9 @@ class Template():
         
         fluxes = []
         for filt_i in filts:    
-            templ_filt = interp(filt_i.wave, self.wave*(1+z),
-                                fnu, left=0, right=0)
+            templ_filt = interp(filt_i.wave.astype(float),
+                                self.wave.astype(float)*(1+z),
+                                fnu.astype(float), left=0, right=0)
                 
             # f_nu/lam dlam == f_nu d (ln nu)    
             integrator = np.trapz
@@ -1155,7 +1306,7 @@ PHOENIX_TEFF = [400.,  420., 450., 500.,  550., 600.,  650., 700.,  750.,
 PHOENIX_ZMET_FULL = [-2.5, -2.0, -1.5, -1.0, -0.5, -0., 0.5]
 PHOENIX_ZMET = [-1.0, -0.5, -0.]
 
-def load_phoenix_stars(logg_list=PHOENIX_LOGG, teff_list=PHOENIX_TEFF, zmet_list=PHOENIX_ZMET, add_carbon_star=True, file='bt-settl_t400-7000_g4.5.fits'):
+def load_phoenix_stars(logg_list=PHOENIX_LOGG, teff_list=PHOENIX_TEFF, zmet_list=PHOENIX_ZMET, add_carbon_star=True, file='bt-settl_t400-7000_g4.5.fits', sonora_dwarfs=True):
     """
     Load Phoenix stellar templates
     
@@ -1180,7 +1331,8 @@ def load_phoenix_stars(logg_list=PHOENIX_LOGG, teff_list=PHOENIX_TEFF, zmet_list
             break
     
     if hdu is None:
-        url = 'https://s3.amazonaws.com/grizli/CONF'
+        #url = 'https://s3.amazonaws.com/grizli/CONF'
+        url = 'https://erda.ku.dk/vgrid/Gabriel%20Brammer/CONF/'
         print('Fetch {0}/{1}'.format(url, file))
 
         #os.system('wget -O /tmp/{1} {0}/{1}'.format(url, file))
@@ -1220,10 +1372,43 @@ def load_phoenix_stars(logg_list=PHOENIX_LOGG, teff_list=PHOENIX_TEFF, zmet_list
 
         tstars.append(Template(arrays=(sp['wave'], cflux), 
                                name='carbon-lancon2002'))
-
+    
+    if sonora_dwarfs:
+        tstars += load_sonora_stars()
+        
     return tstars
 
 
+def load_sonora_stars():
+    """
+    Load Sonora brown dwarf models
+    """
+    import glob
+    
+    paths = ['/tmp', './templates/', './']
+    found = False
+    
+    for path in paths:
+        templ_path = os.path.join(path, 'sonora')
+        if os.path.exists(templ_path):
+            print(f'sonora_stars: {templ_path}')
+            found = True
+            break
+    
+    if not found:
+        return []
+    
+    files = glob.glob(os.path.join(templ_path, 'sonora*fits'))
+    files.sort()
+    
+    stars = []
+    for file in files:
+        name = ' ' + os.path.basename(file).split('.fits')[0]
+        stars.append(Template(file=file, name=name))
+
+    return stars
+
+
 def param_table(templates):
     """
     Try to generate parameters for a list of templates from their 
